import koji
import fedmsg
import fedmsg.encoding

from jobtriggers import config, utils, JobTrigger
from jobtriggers import exceptions as exc

ITEM_TYPE = 'koji_build'


def get_nvr(msg):
    release = msg['msg']['release']
    state = msg['msg']['new']
    instance = msg['msg']['instance']

    name = msg['msg']['name']
    version = msg['msg']['version']

    nvr = '-'.join((name, version, release))

    if not instance == 'primary':
        raise exc.TriggerMsgError('rejecting message not from primary koji instance (%s from %s)' %
                                  (nvr, instance))

    if state != koji.BUILD_STATES['COMPLETE']:
        raise exc.TriggerMsgError('rejecting message for not being a build complete message '
                                  '(%s, status %s)' % (nvr, state))

    if 'fc' not in release:
        raise exc.TriggerMsgError('rejecting message for not containing "fc" in release (%s)' %
                                  nvr)

    return name, version, release


def trigger_tasks(msg, f):
    name, version, release = get_nvr(msg)

    # run dist-git package-specific
    branch = utils.get_branch(release)
    repo = utils.get_repo(name)
    distgit_tasks = utils.get_distgit_tasks(name, ITEM_TYPE, repo, branch)
    f('-'.join((name, version, release)),
      ITEM_TYPE,
      distgit_tasks,
      config.valid_arches,
      branch=branch,
      repo=repo)

    # run general qa tasks
    f('-'.join((name, version, release)),
      ITEM_TYPE,
      config.koji_build_completed_tasks,
      config.valid_arches)

    # run docker-specific tasks here - e. g. dockerautotest
    if name == "docker":
        f('-'.join((name, version, release)),
          ITEM_TYPE,
          config.docker_build_tasks,
          config.valid_arches)

    # run libabigail task here
    critpath = utils.parse_yaml_from_file(config.critpath_filepath)
    critpath_pkgs = critpath['pkgs'].get(branch, critpath['pkgs']['master'])
    libabigail_whitelist = set(critpath_pkgs) - set(config.critpath_exclude)

    if name in libabigail_whitelist:
        f('-'.join((name, version, release)),
          ITEM_TYPE,
          ['libabigail'],
          config.valid_arches)


class KojiBuildCompletedJobTrigger(JobTrigger):
    topic = "org.fedoraproject.prod.buildsys.build.state.change"
    jsonify = False
    config_key = 'taskotron.kojibuildcompletedjobtrigger.enabled'

    def __init__(self, *args, **kw):
        super(KojiBuildCompletedJobTrigger, self).__init__(*args, **kw)

    def consume(self, message):
        msg = fedmsg.encoding.loads(message.body)

        try:
            trigger_tasks(msg, self.trigger_tasks)
        except exc.TriggerMsgError, e:
            self.log.debug(e)
        except exc.TriggerError, e:
            self.log.error(e)
